package com.owlmaddie.damage;

import net.minecraft.entity.damage.DamageSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.UUID;

/**
 * The {@code DamageTracker} class is used to track recent damage to players (type and source of damage).
 */
public class DamageTracker {
    public static final Logger LOGGER = LoggerFactory.getLogger("creaturechat");
    private final Map<UUID, DamageInfo> damageMap = new HashMap<>();
    private final int ticksToTrack;

    public DamageTracker(int ticksToTrack) {
        this.ticksToTrack = ticksToTrack;
    }

    public synchronized void recordDamage(UUID playerUuid, DamageSource damageSource) {
        DamageInfo damageInfo = new DamageInfo(damageSource, ticksToTrack);
        damageMap.put(playerUuid, damageInfo);
        LOGGER.info("Damage recorded to player");
    }

    public synchronized DamageInfo getDamageInfo(UUID playerUuid) {
        return damageMap.get(playerUuid);
    }

    public synchronized void tick() {
        Iterator<Map.Entry<UUID, DamageInfo>> iterator = damageMap.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<UUID, DamageInfo> entry = iterator.next();
            DamageInfo damageInfo = entry.getValue();
            damageInfo.decrementTicks();
            if (damageInfo.getTicksLeft() <= 0) {
                LOGGER.info("Damage cleared from player");
                iterator.remove();
            }
        }
    }

    public static class DamageInfo {
        private final DamageSource damageSource;
        private int ticksLeft;

        public DamageInfo(DamageSource damageSource, int ticksLeft) {
            this.damageSource = damageSource;
            this.ticksLeft = ticksLeft;
        }

        public DamageSource getDamageSource() {
            return damageSource;
        }

        public int getTicksLeft() {
            return ticksLeft;
        }

        public void decrementTicks() {
            this.ticksLeft--;
        }

        public void resetTicks(int ticksLeft) {
            this.ticksLeft = ticksLeft;
        }
    }
}
